@@ -12,7 +12,7 @@ from astropy.wcs import wcs
 from astropy.coordinates import SkyCoord
 import astropy.units as u
 
-from photutils import make_source_mask
+from photutils.segmentation import make_source_mask
 from astropy.stats import sigma_clipped_stats
 from photutils.background import Background2D, MedianBackground
 from photutils.segmentation import detect_sources
@@ -133,38 +133,6 @@ def phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
                      band,
                    filepath=patht[0:-3])
 
-#    path = f'tables/{gal}_basic.tab'
-#    patht = 'tables/{}_basic.tab'.format(gal[:-1])
-#    if os.path.exists(path)  or os.path.exists(patht):
-#        print (f'\nthe table {path} already exist')
-#        try:
-#            tab0 = Table.read(path, format='ascii')
-#        except:
-#            tab0 = Table.read(patht, format='ascii')
-#    else:
-#        query_txt = ("SELECT det.ID, det.RA, det.DEC, det.A, det.B, det.Theta" +
-#                     ", det.KRON_rADIUS, det.CLASS_STAR, det.FIELD, claf.CLASS"+
-#                     ", claf.model_flag, claf.PROB_GAL, claf.PROB_QSO, " +
-#                     "claf.PROB_STAR  FROM idr3.all_idr3 as det join"+
-#                     " idr3.vac_star_galaxy_quasar as claf on "+
-#                     "claf.ID = det.ID WHERE 1 = CONTAINS( POINT('ICRS', " +
-#                     "det.ra, det.dec), CIRCLE('ICRS'," +
-#                     str(tab_gal['ra'][idg]) + "," + str(tab_gal['dec'][idg]) +
-#                     ","+str(10/3600.)+"))")
-#
-#        print ("\nDoing the following query ...\n", query_txt)
-#        tab0=conn.query(query_txt)
-#
-#        tab0.write(f'tables/{gal}_basic.tab', format='ascii')
-
-
-
-#    ZP = Table.read('zero_points.fits')
-#    id_zp = np.where(ZP['field']==tab0['Field'][0])[0][0]
-#    zp = ZP[band][id_zp]
-#    print ('\nid_zp_table:',id_zp,'ZP',zp,'Field:', tab0['Field'][0])
-
-
     Field = out['field']
     #DR_array_all.append(out['dr'])
     print('#############################')
@@ -333,17 +301,8 @@ def phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
         segm = deblend_sources(imag, segm, npixels=npixels_g,
                                 #kernel=kernel,
                                 nlevels=32, contrast=0.001)
-
-    #kerner_size = 20
-    #segmap_float = ndi.uniform_filter(np.float64(segm.data), size=kerner_size)
-    #segmap = segmap_float > 0.5
-
-
-    #gal_main = np.argmax(segm.areas)
-    #print( gal_main )
-    ##print(segm.areas)
-    #print(np.argsort(segm.areas))
-
+    ######################################
+    # Plotting the final segmentation map
 
     print ("\nFinal Segmentation map:")
     print ("-------------------------")
@@ -375,7 +334,7 @@ def phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
     print('\n\nResults')
     print('-----------\n')
 
-
+    w =  wcs.WCS(imagn[1].header)
     if gal[0:2]=='AM' and gal[-1].isalpha():
         gal = gal[:-1]
 
@@ -393,11 +352,15 @@ def phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
            ind=np.where(data['GAL']==gal+listl[i])[0][0]
            indgal =np.where(tab_gal['gal']==gal)[0][0]
 
-           data['ra'][ind] = tab_gal['ra'][indgal]
-           data['dec'][ind] = tab_gal['dec'][indgal]
+           coord_gal = w.all_pix2world(data['Ser_xc'][ind],
+                                     data['Ser_yc'][ind], 0)
+           data['ra'][ind] = coord_gal[0]
+           data['dec'][ind] = coord_gal[1]
+           #data['ra'][ind] = tab_gal['ra'][indgal]
+           #data['dec'][ind] = tab_gal['dec'][indgal]
            data['radvel'][ind] = tab_gal['vel'][indgal]
            try:
-               data['z'][ind] = tab_gal['Z'][indgal]
+               data['z'][ind] = tab_gal['z'][indgal]
            except:
                data['z'][ind] = tab_gal['vel'][indgal]/c_speed
            try:
@@ -416,12 +379,15 @@ def phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
 
            ind=np.where(data['GAL']==gal + user_list[i])[0][0]
            indgal =np.where(tab_gal['gal']==gal)[0][0]
-
-           data['ra'][ind] = tab_gal['ra'][indgal]
-           data['dec'][ind] = tab_gal['dec'][indgal]
+           coord_gal = w.all_pix2world(data['Ser_xc'][ind],
+                                       data['Ser_yc'][ind], 0)
+           data['ra'][ind] = coord_gal[0]
+           data['dec'][ind] = coord_gal[1]
+           #data['ra'][ind] = tab_gal['ra'][indgal]
+           #data['dec'][ind] = tab_gal['dec'][indgal]
            data['radvel'][ind] = tab_gal['vel'][indgal]
            try:
-               data['z'][ind] = tab_gal['Z'][indgal]
+               data['z'][ind] = tab_gal['z'][indgal]
            except:
                data['z'][ind] = tab_gal['vel'][indgal]/c_speed
            try:
@@ -452,7 +418,7 @@ def plot_stat(morph, gal, imag, imagm, median_sky_g, area, t, zp, psfg, psf_g,
         datat = ascii.read(direcbase + '/properties.dat')
         print (Fore.RED + '\n*It was create the properties.dat to print the out parameters')
 
-        unitst = [str, np.float64, np.float64, np.float64, np.float64, np.float64,
+        unitst = [str, np.float64, np.float64, np.float64, np.float64, str,
                  np.float64, np.float64, np.float64, np.float64, np.float64,
                  np.float64, np.float64, np.float64, np.float64, np.float64,
                  np.float64, np.float64, np.float64, np.float64, np.float64,
@@ -476,7 +442,8 @@ def plot_stat(morph, gal, imag, imagm, median_sky_g, area, t, zp, psfg, psf_g,
         print (Fore.BLACK +'\nRow was created for {}'.format(gal))
         data.add_row()
         ind = -1
-        if len(gal) > len(data['GAL'][0]):
+        len_ar = np.array([len(s) for s in data['GAL']])
+        if len(gal) > np.max(len_ar):
             print ('Change the string format of GAL column')
             #data['GAL'].asdtype = 'str'+str(len(gal))
             new_c = data['GAL'].astype('U'+str(len(gal)))
@@ -627,7 +594,7 @@ def plot_stat(morph, gal, imag, imagm, median_sky_g, area, t, zp, psfg, psf_g,
 
 def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
                  region='circle', server=None, limtvel=500, xtpos=10, ytpos=20,
-                 deltay=10, sky_box=[100, 100], deblend='off', psf=1,
+                 deltay=10, ymin = 20, sky_box=[100, 100], deblend='off', psf=1,
                  usesimbad='no', band='R', field_size_phy=100, nsigma=2,
                  npixels=5, dilate_size=11, eta=0.2, show_plot='yes', run_auto='no',
                  outputfile='properties.dat'):
@@ -700,20 +667,12 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
 
     print(Fore.BLACK +'')
 
+    ##################################
+    # Indentifying objects with simbad
     if run_auto=='no':
         outsimbad = cat_simbad (gal, tab_gal, size_image_phy=radsimbad, region=region,
                             limtvel=limtvel, server=server)
 
-
-    # Indentifying objects with simbad
-
-    #radsimbad = 100
-    #if size_image_phy > radsimbad:
-    #    radsimbad = size_image_phy
-    #outsimbad = cat_simbad (gal, tab_gal, size_image_phy=radsimbad, region=region,
-    #                        limtvel=limtvel, server=server)
-
-
     print(Fore.BLACK +'\n\nLoading/Downloading the images')
     print('---------------------------------------------')
 
@@ -782,8 +741,6 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
             make_lupton_rgb(imag_I, imag, imag_G, stretch=0.5, Q=8,
                             filename=path)
 
-
-
     else:
         print (f'\nThe image {path} already exist')
 
@@ -791,7 +748,8 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
     if run_auto=='no':
         plot_simbad(gal, tab_gal, conn, size_image_phy=radsimbad, table=outsimbad,
                 add_object=False, xtpos=xtpos, ytpos=ytpos, deltay=deltay,
-                print_arp='no', print_scale='no', usesimbad=usesimbad, band=band)
+                ymin = ymin, print_arp='no', print_scale='no',
+                usesimbad=usesimbad, band=band)
 
     readn = 'images/fig_{}_{}kpc.png'.format(gal, size_image_phy)
     ima50 = plt.imread(readn)
@@ -881,9 +839,6 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
                                #kernel=kernel,
                                 nlevels=32, contrast=0.001)
 
-    #plt.imshow(segm_deblend, origin='lower')
-
-
     gal_main = np.argmax(segm.areas)
     print("\nArea in pixel of the detection objects: ")
     print(segm.areas)
@@ -903,17 +858,13 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
     mpl.rcParams['xtick.labelsize']= 16
     mpl.rcParams['ytick.labelsize']= 16
 
+    ###########################
+    # Plotting Segmentation map
+
     fig=plt.figure(figsize=(8,8))
     segmc = np.copy(segm.data).astype(float)
     segmc[segmc==0.0]=np.nan
     size_image_kpc = int(size_image)*0.5*0.55*arc_kpc
-    # x = np.linspace(size_image_kpc, size_image_kpc, segmc.shape[0])
-    # y = np.linspace(size_image_kpc, size_image_kpc, segmc.shape[1])
-    #
-    # X, Y = np.meshgrid(x, y)
-    # imseg = plt.contourf(X, Y, segmc, cmap='tab20', origin='lower',
-    #         nlevels=10,
-    #         extent=np.array([-1,1,-1,1])*size_image_kpc)
 
     imseg = plt.imshow(segmc, cmap='Paired', interpolation='nearest',
                        origin='lower')
@@ -930,7 +881,6 @@ def ex_phot_mod (gal, tab_gal, conn, size_image_phy=50, snr_g = 2.0, areagal=2,
         plt.show()
     else:
         plt.close()
-    #fits.writeto('test.fits', segmc, overwrite=True)
 
     if show_plot=='yes':
         if usesimbad=='no':
@@ -1088,7 +1038,7 @@ def simbad_download(GAL, tab_gal, size_image_phy=100,  width = 300,
 
 
 def simbad_thumbnail(GAL, tab_gal, size_image_phy=100,  width = 300,
-                     height = 300,  xtpos=10, ytpos=20, deltay=10,
+                     height = 300,  xtpos=10, ytpos=20, deltay=10, miny=20,
                      labeltxt = 'off', limtvel=500, ned_plot='no',
                      deltay_ned=12):
 
@@ -1223,12 +1173,12 @@ def simbad_thumbnail(GAL, tab_gal, size_image_phy=100,  width = 300,
 
             else:
                 yposb = Ygc[ordem][cond][i-1] + ytpos
-                if abs(ypos - yposb)<30:
+                if abs(ypos - yposb)<miny:
                     #print ('hola', i)
                     ypos+=deltay
                 if i>1:
                     yposbb = Ygc[ordem][cond][i-2] + ytpos
-                    if abs(ypos - yposbb)<60:
+                    if abs(ypos - yposbb)<miny*2:
                         ypos+=deltay
 
                 plt.text(xpos, ypos, textl, fontsize=12, color='w')
@@ -1248,12 +1198,12 @@ def simbad_thumbnail(GAL, tab_gal, size_image_phy=100,  width = 300,
 
             else:
                 yposb = Ygc_ned[ordem][cond_ned][i-1] + ytpos - deltay_ned
-                if abs(ypos - yposb)<30:
+                if abs(ypos - yposb)<miny:
                     #print ('hola', i)
                     ypos+=deltay
                 if i>1:
                     yposbb = Ygc_ned[ordem][cond_ned][i-2] + ytpos - deltay_ned
-                    if abs(ypos - yposbb)<60:
+                    if abs(ypos - yposbb)<miny*2:
                         ypos+=deltay
 
                 plt.text(xpos, ypos, textl, fontsize=12, color='r')
@@ -1266,10 +1216,10 @@ def simbad_thumbnail(GAL, tab_gal, size_image_phy=100,  width = 300,
 
 
 def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
-                user_text=None, xtpos=10, ytpos=20, deltay=10,
+                user_text=None, xtpos=10, ytpos=20, deltay=10, miny=20,
                 savefig=None, add_object=True, itsconsider='no', itsgroup='off',
                 Nmembers=0, print_arp='yes', print_scale='yes',
-                usesimbad = 'no', band='R'):
+                usesimbad = 'no', band='R', outputfile='properties.dat'):
 
     idg =np.where(tab_gal['gal']==gal)[0][0]
 
@@ -1298,9 +1248,12 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
     if add_object:
         data = ascii.read(outputfile)
 
-
-        skyc=SkyCoord(table['RA_d'][user_order], table['DEC_d'][user_order])
+        ind_inters = np.intersect1d(data['GAL'], user_text, return_indices=True)
+        skyc=SkyCoord(data['ra'][ind_inters[1]]*u.deg,
+                      data['dec'][ind_inters[1]]*u.deg)
         sep = skyc[0].separation(skyc[1::])
+        #skyc=SkyCoord(table['RA_d'][user_order], table['DEC_d'][user_order])
+        #sep = skyc[0].separation(skyc[1::])
 
         sep_kpc = sep.arcsec*arc_kpc
 
@@ -1328,10 +1281,11 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
                 ind = -1
                 data['GAL'][ind] = comp
 
-            data['ra'][ind] = table['RA_d'][comp_ind]
-            data['dec'][ind] = table['DEC_d'][comp_ind]
+            #data['ra'][ind] = table['RA_d'][comp_ind]
+            #data['dec'][ind] = table['DEC_d'][comp_ind]
             data['radvel'][ind] = table['RV_VALUE'][comp_ind]
             data['z'][ind] = table['Z_VALUE'][comp_ind]
+            data['morph_type'] = data['morph_type'].astype(str)
             data['morph_type'][ind] = table['MORPH_TYPE'][comp_ind]
             if itsconsider=='yes':
                 data['Its_considered?'][ind]=1
@@ -1447,7 +1401,6 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
         plt.scatter(Xgc[cond], Ygc[cond], s=100, facecolors='none',
                     edgecolors='g')
 
-
         for i, indg in enumerate(ordem[cond]):
             textl = table['OTYPE'][indg] + ' Vrad={:.0f} ({}) B={:.1f}'.format(
                     table['RV_VALUE'][indg], indg, table['FLUX_B'][indg])
@@ -1460,23 +1413,15 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
 
             else:
                 yposb = Ygc[ordem][cond][i-1]
-                if ypos - yposb<30:
+                if abs(ypos - yposb)<miny:
                     ypos+=deltay
                 if i>1:
                     yposbb = Ygc[ordem][cond][i-2]
-                    if ypos - yposbb<60:
+                    if abs(ypos - yposbb)<miny*2:
                         ypos+=deltay
 
                 plt.text(xpos, ypos, textl, fontsize=12, color='w')
 
-
-            # plt.annotate(textl, # this is the text
-            #              (Xgc[cond][i], ima.shape[0]-Ygc[cond][i]),
-            # these are the coordinates to position the label
-            #              textcoords="offset points", # how to position the text
-            #              xytext=(0,10), # distance from text to points (x,y)
-            #              ha='center', color='r') # h
-
         plt.minorticks_on()
         plt.ylabel('$\Delta\delta$ (arcsec)')
         plt.xlabel('$\Delta \\alpha$ (arcsec)')
@@ -1484,39 +1429,46 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
 
 
     else:
-        plt.scatter(Xgc[user_order], Ygc[user_order], s=100, facecolors='none',
+        ind_inters = np.intersect1d(data['GAL'], user_text, return_indices=True)
+        Xgc, Ygc = w.all_world2pix(data['ra'][ind_inters[1]]*u.deg,
+                                    data['dec'][ind_inters[1]]*u.deg, 0)
+
+        Xgc = (Xgc*pix_scima - size_image*0.5*pix_scima)*-1
+        Ygc = Ygc*pix_scima - size_image*0.5*pix_scima
+
+        cond = ( (Xgc>ima.shape[1]*-0.5*pix_scima) & (Ygc>ima.shape[0]*-0.5*pix_scima) &
+                 (Xgc<0.5*pix_scima*ima.shape[1]) & (Ygc<0.5*pix_scima*ima.shape[0]))
+
+        #plt.scatter(Xgc[user_order], Ygc[user_order], s=100, facecolors='none',
+        #            edgecolors='g')
+        plt.scatter(Xgc, Ygc, s=100, facecolors='none',
                     edgecolors='g')
 
 
         for i, indg in enumerate(user_order):
             textl = user_text[i]
 
-
-            xpos = Xgc[indg] + xtpos
-            ypos = Ygc[indg] + ytpos
+            #xpos = Xgc[indg] + xtpos
+            #ypos = Ygc[indg] + ytpos
+            xpos = Xgc[i] + xtpos
+            ypos = Ygc[i] + ytpos
 
             if i==0:
                 plt.text(xpos, ypos, textl, fontsize=12, color='w')
-
             else:
-                yposb = Ygc[user_order][i-1]
-                if ypos - yposb<30:
+                #yposb = Ygc[user_order][i-1]
+                yposb = Ygc[-1]
+
+                if abs(ypos - yposb)<miny:
                     ypos+=deltay
                 if i>1:
-                    yposbb = Ygc[user_order][i-2]
-                    if ypos - yposbb<60:
+                    #yposbb = Ygc[user_order][i-2]
+                    yposbb = Ygc[i-2]
+                    if abs(ypos - yposbb)<miny*2:
                         ypos+=deltay
 
                 plt.text(xpos, ypos, textl, fontsize=12, color='w')
 
-
-            # plt.annotate(textl, # this is the text
-            #              (Xgc[cond][i], ima.shape[0]-Ygc[cond][i]),
-            # these are the coordinates to position the label
-            #              textcoords="offset points", # how to position the text
-            #              xytext=(0,10), # distance from text to points (x,y)
-            #              ha='center', color='r') # h
-
         plt.minorticks_on()
         plt.ylabel('$\Delta\delta$ (arcsec)')
         plt.xlabel('$\Delta \\alpha$ (arcsec)')
@@ -1524,7 +1476,8 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
             print ("\nSaving figure in images/{}.png".format(gal))
             plt.savefig('images/{}.png'.format(gal), bbox_inches='tight')
         #plt.show()
-        Xgc, Ygc = w.all_world2pix(table['RA_d'][0], table['DEC_d'][0], 0)
+        Xgc, Ygc = w.all_world2pix(tab_gal['ra'][idg]*u.deg,
+                                   tab_gal['dec'][idg]*u.deg, 0)
 
         if usesimbad == 'yes':
             path = 'fits_images/{}_simbad_{}kpc.fits'.format(gal, size_image_phy)
@@ -1539,7 +1492,7 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
         print ("ima = fits.open('{}')[1]".format(path))
         print ("w =  wcs.WCS(ima.header)")
         print ("coord_a = SkyCoord('{} {}', unit=u.deg)".format(
-               table['RA_d'][0], table['DEC_d'][0]))
+               data['ra'][ind_inters[1][0]], data['dec'][ind_inters[1][0]]))
         print ("coord_b = w.all_pix2world({} + delta_ra/{}*-1,".format(Xgc,
                                                                     pix_scima))
         print ("                          {} + delta_dec/{}, 0)".format(Ygc,
@@ -1566,7 +1519,8 @@ def plot_simbad(gal, tab_gal, conn, size_image_phy, table, user_order=None,
 def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
                         areagal=2, band='r', sky_box=[100, 100], deblend='off',
                         psf=1, run_auto='no', show_plot='yes',
-                        nsigma=2, npixels=5, dilate_size=11):
+                        nsigma=2, npixels=5, dilate_size=11,
+                        outputfile='properties.dat'):
 
     idg =np.where(tab_gal['gal']==gal)[0][0]
     D = tab_gal['vel'][idg]/H
@@ -1626,7 +1580,8 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
 
     readn = 'images/fig_{}_{}kpc.jpeg'.format(gal, size_image_phy)
     ima50 = plt.imread(readn)
-    imagn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal, band, size_image_phy))
+    imagn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal, band,
+                      size_image_phy))
     imag = imagn[0].data
 
     size_image_kpc = int(size_image)*0.5*0.55*arc_kpc
@@ -1666,8 +1621,6 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
     print('median_sky_g = {:.2f}'.format(median_sky))
     print('std_sky_g = {:.2f}'.format(std_sky))
 
-
-
     print(('\n\nEstimating the 2D sky _background by using a sky box' +
             ' of {}x{} pixel').format(sky_box[0],sky_box[1]))
     print('------------------------------------------')
@@ -1712,8 +1665,6 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
             plt.show()
         else:
             plt.close()
-    #plt.imshow(segm_deblend, origin='lower')
-
 
     gal_main = np.argmax(segm.areas)
     print("\nArea in pixel of the detection objects: ")
@@ -1734,21 +1685,15 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
     mpl.rcParams['xtick.labelsize']= 16
     mpl.rcParams['ytick.labelsize']= 16
 
+    ###########################
+    # Plotting Segmentation map
+
     fig=plt.figure(figsize=(8,8))
     segmc = np.copy(segm.data).astype(float)
     segmc[segmc==0.0]=np.nan
 
-    # x = np.linspace(size_image_kpc, size_image_kpc, segmc.shape[0])
-    # y = np.linspace(size_image_kpc, size_image_kpc, segmc.shape[1])
-    #
-    # X, Y = np.meshgrid(x, y)
-    # imseg = plt.contourf(X, Y, segmc, cmap='tab20', origin='lower',
-    #         nlevels=10,
-    #         extent=np.array([-1,1,-1,1])*size_image_kpc)
-
     imseg = plt.imshow(segmc, cmap='Paired', interpolation='nearest',
                        origin='lower')
-                       #extent=np.array([-1,1,-1,1])size_image_kpc)
 
     for i in np.arange(np.nanmax(segmc)):
        pos=np.where(segmc==(i+1))
@@ -1761,14 +1706,13 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
         plt.show()
     else:
         plt.close()
-    fits.writeto('test.fits', segmc, overwrite=True)
 
     if run_auto == 'no':
 
         part0 = f"\n\nSL.phot_mod_decals('{gal}', tab, size_image_phy={size_image_phy}, "
         part1 = f""" snr_g = {snr_g}, areagal={areagal}, band="{band}", """
         part2 = f"""sky_box = {sky_box},\n deblend ="{deblend}", """
-        part3 = """user_order=[1,2], user_list=["A","B"])"""
+        part3 = """user_order=[1,2], user_list=["A","B"], outputfile={outputfile=})"""
         print(part0 + part1 + part2 + part3)
     else:
 
@@ -1777,13 +1721,13 @@ def ex_phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0,
         part0 = f"\n\nSL.phot_mod_decals('{gal}', tab, size_image_phy={size_image_phy}, "
         part1 = f""" snr_g = {snr_g}, areagal={areagal}, band="{band}", """
         part2 = f"""sky_box = {sky_box},\n deblend ="{deblend}", """
-        part3 = f"""user_order=[{maxarea}], user_list=["{band}"])"""
+        part3 = f"""user_order=[{maxarea}], user_list=["{band}"], outputfile={outputfile=})"""
         print(part0 + part1 + part2 + part3)
 
         phot_mod_decals(gal, tab_gal, size_image_phy=size_image_phy, snr_g = snr_g,
                            areagal=areagal, band=band, sky_box = sky_box,
                            deblend =deblend, user_order=[maxarea], user_list=[band],
-                           show_plot=show_plot)
+                           show_plot=show_plot, outputfile=outputfile)
 
 
 def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
@@ -1803,9 +1747,9 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
     radius_cat = (size_image*0.5*0.55)/3600.
     print (Fore.BLUE + 'image size in pix: {:.1f}'.format(size_image))
     area_min=(((np.square(areagal)*np.pi)/arc_kpc**2)/0.55**2)
-    print (Fore.BLUE +'Minimum detection area {:.1f} kcp^2 = {:.0f} pix'.format(areagal,
-           area_min))
-
+    textt = 'Minimum detection area {:.1f} kcp^2 = {:.0f} pix'.format(areagal,
+           area_min)
+    print (Fore.BLUE + textt)
 
     print(Fore.BLACK + '\n\nZP and mean PSF of the Field')
     print('--------------------------------')
@@ -1824,10 +1768,11 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
     t=1
 
 
-
-
-    imagn = fits.open('fits_images/' + tab_gal['gal'][idg]+'_{}_{}kpc.fits'.format(
-                      band, size_image_phy))
+    ###################
+    # Loading the image
+    patht = 'fits_images/' + tab_gal['gal'][idg]+'_{}_{}kpc.fits'.format(
+            band, size_image_phy)
+    imagn = fits.open(patht)
     imag = imagn[0].data
 
     ##################
@@ -1878,8 +1823,10 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
         plt.close()
 
     pathgal = tab_gal['gal'][idg]+'_{}_{}kpc'.format(band, size_image_phy)
-    fits.writeto('fits_images/' + pathgal + '_sky.fits', bkg.background, overwrite=True)
-    fits.writeto('fits_images/' + pathgal + '_sky_sub.fits', imag, overwrite=True)
+    fits.writeto('fits_images/' + pathgal + '_sky.fits', bkg.background,
+                 overwrite=True)
+    fits.writeto('fits_images/' + pathgal + '_sky_sub.fits', imag,
+                 overwrite=True)
     print ("Saving " + 'fits_images/' + pathgal + '_sky_sub.fits...\n')
 
 
@@ -1889,7 +1836,6 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
     ##################
     ## PSF
 
-    #psf_g = psf_g/sc_imag
     fac_sig=np.sqrt(np.log(2)*2)*2
     size = 20  # on each side from the center
     sigma_psf = psf_g/(fac_sig)
@@ -1903,7 +1849,6 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
     sigma = psf_g * gaussian_fwhm_to_sigma
     kernel = Gaussian2DKernel(sigma, x_size=3, y_size=3)
     kernel.normalize()
-    #fits.write('kernel.fits', kernel, overwrite=True)
 
     npixels_g = int(area_min) # minimum number of connected pixels
     threshold = photutils.detect_threshold(imag, nsigma=snr_g)
@@ -1924,23 +1869,9 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
                                 #kernel=kernel,
                                 nlevels=32, contrast=0.001)
 
-    #kerner_size = 20
-    #segmap_float = ndi.uniform_filter(np.float64(segm.data), size=kerner_size)
-    #segmap = segmap_float > 0.5
-
-
-    #gal_main = np.argmax(segm.areas)
-    #print( gal_main )
-    ##print(segm.areas)
-    #print(np.argsort(segm.areas))
-
-
     print ("\nFinal Segmentation map:")
     print ("-------------------------")
 
-    #plt.figure(figsize=(6.5,6.5))
-    #plt.imshow(segm, origin='lower', cmap='gray')
-
     fig=plt.figure(figsize=(8,8))
     segmc = np.copy(segm.data).astype(float)
     segmc[segmc==0.0]=np.nan
@@ -1962,9 +1893,6 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
     else:
         plt.close()
 
-
-    #segmap=np.copy(segm.data)
-
     print(Fore.RED + '\n\nRunning Statmorph')
     print('---------------------')
 
@@ -1996,7 +1924,7 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
            data['ra'][ind] = tab_gal['ra'][indgal]
            data['dec'][ind] = tab_gal['dec'][indgal]
            data['radvel'][ind] = tab_gal['vel'][indgal]
-           data['z'][ind] = tab_gal['Z'][indgal]
+           data['z'][ind] = tab_gal['z'][indgal]
            data['Jclass'][ind] = tab_gal['Jclass'][indgal]
 
            ascii.write(data, outputfile, overwrite=True)
@@ -2015,60 +1943,48 @@ def phot_mod_decals (gal, tab_gal, size_image_phy=50, snr_g = 2.0, areagal=2,
            data['ra'][ind] = tab_gal['ra'][indgal]
            data['dec'][ind] = tab_gal['dec'][indgal]
            data['radvel'][ind] = tab_gal['vel'][indgal]
-           data['z'][ind] = tab_gal['Z'][indgal]
+           data['z'][ind] = tab_gal['z'][indgal]
            data['Jclass'][ind] = tab_gal['Jclass'][indgal]
 
            ascii.write(data, outputfile, overwrite=True)
 
     plt.show()
 
-def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat'):
+def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat',
+                suf_num=-1, show_plot='yes'):
 
     import functions as func
-    idg =np.where(tab_gal['gal']==gal[:-1])[0][0]
+
+    idg =np.where(tab_gal['gal']==gal[:suf_num])[0][0]
     D = tab_gal['vel'][idg]/H
-    z_C = tab_gal['z'][idg]
-
-    D_gal = (c_speed*z_C)/H
-    delta_D = abs(D-D_gal)*1000
-
     arc_kpc = np.tan(np.deg2rad(1./3600.))*D*1e3
     sc_ima=0.27
     size_image=((size_image_phy/arc_kpc)/sc_ima)*2
 
-    ra=np.array(tab_gal['RA'])
-
-    if ra[idg]!=0.0:
-        coord_gal = SkyCoord(tab_gal['ra'][idg], tab_gal['dec'][idg], unit=u.deg)
-        coord_C = SkyCoord(tab_gal['RA'][idg], tab_gal['DEC'][idg], unit=u.deg)
-        sep = coord_gal.separation(coord_C)
-
-        sep_kpc = sep.arcsec*arc_kpc
-        D_int = np.sqrt(np.square(sep_kpc)+ np.square(delta_D))
-    else:
-        sep_kpc = np.nan
-        D_int = np.nan
-
     print(Fore.BLACK +'\n\nLoading/Downloading Decals images')
     print('---------------------------------------------')
 
     home = 'https://www.legacysurvey.org/viewer/fits-cutout?'
-    path = 'fits_images/{}_{}_{}kpc.fits'.format(gal[:-1], 'g', size_image_phy)
+    path = 'fits_images/{}_{}_{}kpc.fits'.format(gal[:suf_num], 'g',
+            size_image_phy)
     if not os.path.exists(path):
         print (f"\nDowloading {path} ...")
-        part0 = 'ra={}&dec={}&layer=dr8-south&pixscale=0.27&bands=r&size={}'.format(
+        part0 = 'ra={}&dec={}&layer=dr8&pixscale=0.27&bands=g&size={}'.format(
              tab_gal['ra'][idg], tab_gal['dec'][idg], int(size_image))
         targ = home + part0
         print (targ)
+
+        #https://www.legacysurvey.org/viewer/fits-cutout?ra=153.445175&dec=-0.9142411&layer=ls-dr10-early-grz&pixscale=0.27&bands=g&size=420
         #wget.download(targ, out=path)
         request.urlretrieve(targ, path)
     else:
        print (f'\nThe image {path} already exist')
 
-    path = 'fits_images/{}_{}_{}kpc.fits'.format(gal[:-1], 'r', size_image_phy)
+    path = 'fits_images/{}_{}_{}kpc.fits'.format(gal[:suf_num], 'r',
+           size_image_phy)
     if not os.path.exists(path):
         print (f"\nDowloading {path} ...")
-        part0 = 'ra={}&dec={}&layer=dr8-south&pixscale=0.27&bands=r&size={}'.format(
+        part0 = 'ra={}&dec={}&layer=dr8&pixscale=0.27&bands=r&size={}'.format(
              tab_gal['ra'][idg], tab_gal['dec'][idg], int(size_image))
         targ = home + part0
         print (targ)
@@ -2077,15 +1993,15 @@ def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat'):
     else:
         print (f'\nThe image {path} already exist')
 
-    imagn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal[:-1], 'g',
+    imagn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal[:suf_num], 'g',
                       size_image_phy))
     imag = imagn[0].data
 
-    imaxy = pfunc.callf('fits_images/{}_{}_{}kpc.fits'.format(gal[:-1], 'g',
+    imaxy = pfunc.callf('fits_images/{}_{}_{}kpc.fits'.format(gal[:suf_num],'g',
                       size_image_phy))[0]
 
 
-    imarn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal[:-1], 'r',
+    imarn = fits.open('fits_images/{}_{}_{}kpc.fits'.format(gal[:suf_num], 'r',
                       size_image_phy))
     imar = imarn[0].data
 
@@ -2102,12 +2018,14 @@ def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat'):
 
     gal_pol_1Reff = pfunc.ellipse(x0=centx_phot-0.5, y0=centy_phot-0.5,
                      sma=smaellip, eps=eps0, pa=theta+90,
-                     step=int(func.peri_ellip_exact(smaellip, smaellip*(1-eps0))*2))
+                     step=int(func.peri_ellip_exact(smaellip,
+                              smaellip*(1-eps0))*2))
 
     smaellip = smaellip*2
     gal_pol_2Reff = pfunc.ellipse(x0=centx_phot-0.5, y0=centy_phot-0.5,
                      sma=smaellip, eps=eps0, pa=theta+90,
-                     step=int(func.peri_ellip_exact(smaellip, smaellip*(1-eps0))*2))
+                     step=int(func.peri_ellip_exact(smaellip,
+                              smaellip*(1-eps0))*2))
 
     mask_gal1=pfunc.maskpoly(gal_pol_1Reff, imaxy, imag*np.nan)
     mask_gal2=pfunc.maskpoly(gal_pol_2Reff, imaxy, imag*np.nan)
@@ -2123,7 +2041,12 @@ def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat'):
     plt.xlabel('x (pix)')
     plt.ylabel('y (pix)')
     plt.minorticks_on()
-    plt.show()
+    plt.savefig('images/fig_{}_g_{}kpc_2Re.png'.format(gal[:suf_num], 'g',
+                      size_image_phy), bbox_inches='tight')
+    if show_plot == 'yes':
+        plt.show()
+    else:
+        plt.close()
 
     fluxg = np.nansum(imag + mask_gal2)
     fluxr = np.nansum(imar + mask_gal2)
@@ -2135,25 +2058,102 @@ def mass_decals(gal, tab_gal, size_image_phy, outputfile='properties.dat'):
 
     MLg = 10**(-0.499 + 1.519*(mag_g-mag_r))
     Mg = -5.0*np.log10(D*1e6) + 5.0 + mag_g
+    Mr = -5.0*np.log10(D*1e6) + 5.0 + mag_r
     Lg = 10**((1.0/2.5)*(5.11-Mg))
     mass_stellar = Lg*MLg
 
+
+    try:
+        z_C = tab_gal['Z'][idg]
+        D_gal = (c_speed*z_C)/H
+        delta_D = abs(D-D_gal)*1000
+
+        ra=np.array(tab_gal['RA'])
+        if ra[idg]!=0.0:
+            coord_gal = SkyCoord(tab_gal['ra'][idg], tab_gal['dec'][idg],
+                                 unit=u.deg)
+            coord_C = SkyCoord(tab_gal['RA'][idg], tab_gal['DEC'][idg],
+                               unit=u.deg)
+            sep = coord_gal.separation(coord_C)
+
+            sep_kpc = sep.arcsec*arc_kpc
+            D_int = np.sqrt(np.square(sep_kpc)+ np.square(delta_D))
+        else:
+            sep_kpc = np.nan
+            D_int = np.nan
+    except:
+        pass
+
     data = ascii.read(outputfile)
-
     ind=np.where(data['GAL']==gal)[0][0]
 
-    data['mass'][ind] = np.log10(mass_stellar)
-    data['sig_C'][ind] = tab_gal['sigma_C'][idg]
-    data['D_proj'][ind] = sep_kpc
-    data['D_intr'][ind] = D_int
-    data['Lx'][ind] = tab_gal['Lx'][idg]
-
     print (Fore.BLUE +"\nGeneral information:")
     print (Fore.BLUE +"-------------------")
-    print(Fore.RED +'mass (log) ={:.1f}'.format(np.log10(mass_stellar)))
-    print(Fore.BLUE +'Lx (log) = {:.1f}'.format(tab_gal['Lx'][idg]))
-    print(Fore.BLUE +'sigma = {:.1f} km/s'.format(tab_gal['sigma_C'][idg]))
-    print(Fore.BLUE +'D_proj = {:.1f} kpc'.format(sep_kpc))
-    print(Fore.BLUE +'D_intr = {:.1f} kpc'.format(D_int))
+
+    try:
+        data['mass'][ind] = np.log10(mass_stellar)
+        print(Fore.RED +'\nMass (log) = {:.1f}'.format(np.log10(mass_stellar)))
+    except:
+        print(Fore.BLACK +'The column <<mass>> was added to the {}'.format(
+              outputfile))
+        data.add_column(np.zeros((len(data)))*np.nan, name='mass')
+        data['mass'][ind] = np.log10(mass_stellar)
+        print(Fore.RED +'\nMass (log) = {:.1f}'.format(np.log10(mass_stellar)))
+    try:
+        data['mag_r_2Re'][ind] = mag_r
+        print(Fore.RED +'mag_r_2Re = {:.1f}'.format(mag_r))
+    except:
+        print(Fore.BLACK +'The column <<mag_r_2Re>> was added to the {}'.format(
+              outputfile))
+        data.add_column(np.zeros((len(data)))*np.nan, name='mag_r_2Re')
+        data['mag_r_2Re'][ind] = mag_r
+        print(Fore.RED +'mag_r_2Re = {:.1f}'.format(mag_r))
+    try:
+        data['mag_g_2Re'][ind] = mag_g
+        print(Fore.RED +'mag_g_2Re = {:.1f}'.format(mag_g))
+    except:
+        print(Fore.BLACK +'The column <<mag_g_2Re>> was added to the {}'.format(
+              outputfile))
+        data.add_column(np.zeros((len(data)))*np.nan, name='mag_g_2Re')
+        data['mag_g_2Re'][ind] = mag_g
+        print(Fore.RED +'mag_g_2Re = {:.1f}'.format(mag_g))
+    try:
+        data['Mr_2Re'][ind] = Mr
+        print(Fore.RED +'Mr_2Re = {:.1f}'.format(Mr))
+    except:
+        print(Fore.BLACK +'The column <<Mr_2Re>> was added to the {}'.format(
+              outputfile))
+        data.add_column(np.zeros((len(data)))*np.nan, name='Mr_2Re')
+        data['Mr_2Re'][ind] = Mr
+        print(Fore.RED +'Mr_2Re = {:.1f}'.format(Mr))
+    try:
+        data['Mg_2Re'][ind] = Mg
+        print(Fore.RED +'Mg_2Re = {:.1f}'.format(Mg))
+    except:
+        print(Fore.BLACK +'The column <<Mg_2Re>> was added to the {}'.format(
+              outputfile))
+        data.add_column(np.zeros((len(data)))*np.nan, name='Mg_2Re')
+        data['Mg_2Re'][ind] = Mg
+        print(Fore.RED +'Mg_2Re = {:.1f}'.format(Mg))
+    try:
+        data['sig_C'][ind] = tab_gal['sigma_C'][idg]
+        print(Fore.BLUE +'\nLx (log) = {:.1f}'.format(tab_gal['Lx'][idg]))
+    except:
+        pass
+    try:
+        data['D_proj'][ind] = sep_kpc
+        print(Fore.BLUE +'sigma = {:.1f} km/s'.format(tab_gal['sigma_C'][idg]))
+    except:
+        pass
+    try:
+        data['D_intr'][ind] = D_int
+        print(Fore.BLUE +'D_proj = {:.1f} kpc'.format(sep_kpc))
+    except:
+        pass
+    try:
+        data['Lx'][ind] = tab_gal['Lx'][idg]
+        print(Fore.BLUE +'D_intr = {:.1f} kpc'.format(D_int))
+    except:
+        pass
 
     ascii.write(data, outputfile, overwrite=True)
